/***************************************************************************
 *   Copyright (C) 2012 by Daniel Nicoletti                                *
 *   dantti12@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; see the file COPYING. If not, write to       *
 *   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,  *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/

#ifndef CATEGORYMATCHER_H
#define CATEGORYMATCHER_H

#include <QMetaType>
#include <QString>
#include <QStringList>
#include <QList>

#include <kdemacros.h>

class KDE_EXPORT CategoryMatcher
{
public:
    typedef enum {
        And,
        Or,
        Not,
        Term
    } Kind;
    explicit CategoryMatcher(Kind kind = And, const QString &term = QString());
    CategoryMatcher(const CategoryMatcher &other);
    ~CategoryMatcher();
    CategoryMatcher& operator=(const CategoryMatcher &other);

    bool match(const QStringList &categories) const;

    void setChild(const QList<CategoryMatcher> &child);
    QList<CategoryMatcher> child() const;
    QString term() const;
    Kind kind() const;

private:
    Kind m_kind;
    QString m_term;
    QList<CategoryMatcher> m_child;
};

Q_DECLARE_METATYPE(CategoryMatcher)

#endif // CATEGORYMATCHER_H
